{
   muu TrueColor library for TMT Pascal - FLIc player example
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

USES mArea,mClip,mFormat,mSurface,mConsole,mTarga,mc_VESA,mc_Wrap,Nova,CRT,Error,mFli,mfx32;

//{$DEFINE debug_wrapper}  // enable this ONLY for debugging

    // our Surface and Console, clear them just for neatness
VAR testsurface:    Surface := NIL;
    testsurface2:   Surface := NIL;
    testconsole:    Console := NIL;
    testplayer:     muuFlic := NIL;
    testplayer2:    muuFlic := NIL;

    // blurring procedure w/clipping
PROCEDURE blur32(m_source:Surface;m_dest:Surface;m_area:Area);
  VAR s_data,d_data:        pointer;
      s_next:               dword := 0;
      s_line,d_line:        dword := 0;
      put_width,put_height: dword := 0;
      line_position:        dword := 0;
BEGIN
    // clip area with both surfaces
  m_area := Clip(m_area,Area(1,1,m_source^.Width-1,m_source^.Height-1));
  m_area := Clip(m_area,Area(1,1,m_dest^.Width-1,m_dest^.Height-1));

    // determine how much to put
  put_width := (m_area.Right-m_area.Left);
  put_height := (m_area.Bottom-m_area.Top);

  IF (put_width <= 0) OR
     (put_height <= 0) THEN
       Exit;

    // data start positions
  s_data := ptr(ofs(m_source^.Data^)+(m_area.Left+m_area.Top*m_source^.Width) shl 2);
  d_data := ptr(ofs(m_dest^.Data^)+(m_area.Left+m_area.Top*m_dest^.Width) shl 2);

    // next line
  s_line := (m_source^.Width-put_width) shl 2;
  d_line := (m_dest^.Width-put_width) shl 2;

    // one line up/down
  s_next := m_source^.Width shl 2;
  
  ASM
    mov  esi, s_data
    mov  edi, d_data

  @@yloop:
    mov  eax, put_width
    mov  line_position, eax

    xor  ebx, ebx

  @@xloop:
    mov  ecx, 3
    mov  edx, s_next

    // loop through B,G,R and A
  @@pixloop:
    mov  al,  [esi+4]
    mov  bx,  ax

    mov  al,  [esi-4]
    add  bx,  ax

    mov  al,  [esi+edx*1]
    add  bx,  ax

    xor  edx, -1
    add  edx, 1
    mov  al,  [esi+edx*1]
    add  bx,  ax

    shr  bx,  2
    mov  [edi],  bl

    inc  esi
    inc  edi

    dec  ecx
    jnz  @@pixloop

    inc  esi
    inc  edi

    dec  line_position
    jnz  @@xloop

    add  esi, s_line
    add  edi, d_line

    dec  put_height
    jnz  @@yloop
  END;    
END;


    // set textmode
PROCEDURE set_textmode; code;
ASM
  mov  ax,  $03
  int  $10
  ret
END;

        // -------------- START OF MAIN CODE ----------------
begin

  IF (paramcount >= 1) THEN
    new(testplayer,Open(paramstr(1),Stream))
  ELSE
    fail('Usage: FLIC.EXE <flicfile>.<fli|flc> [flicfile2.fli|flc]');

  IF (paramcount = 2) THEN
    new(testplayer2,Open(paramstr(2),Stream));

    // We will use the VBEConsole to display the image, therefore we overload
    // the testconsole type when creating it (creating means allocating)
    // (if debug_wrapper is defined, wrap VBEConsole to wrapper
  {$IFNDEF debug_wrapper}
    new(VBEConsole(testconsole),Create);
  {$ELSE}
    new(TTYConsole(testconsole),Create);
    DECLARE VAR deb: Console;
    BEGIN
      new(VBEConsole(deb),Create);
      testconsole^.Wrap(deb);
    END;
  {$ENDIF}

    // read options into Console, which will be used instead of the default
    // values if specified (here we load options from "mtc.cfg")
  testconsole^.getOptions(mtc_option_file);

    // options are set, and we can safely proceed opening our Console. Here
    // it's called "Image example", which is a suiting name:)
  testconsole^.Open('Image example',testplayer^.Width,testplayer^.Height,Format(Unknown));

    // check if an error occurred when opening, and exit with error if so
  IF (testconsole^.Error) THEN
    fail('Failed to create console');

  testsurface := Surface(testplayer^.Width,testplayer^.Height,Format(BGR8888));
  testplayer^.Looping(true);

  IF (testplayer2 <> NIL) THEN
  BEGIN
    testsurface2 := Surface(testplayer2^.Width,testplayer2^.Height,Format(BGR8888));
    testplayer2^.Looping(true);
    testplayer2^.SetFrame(1);
  END;

  REPEAT
    testplayer^.Frame;
    testplayer^.Draw(0,0,testsurface);

    IF (testplayer2 <> NIL) THEN
    BEGIN
      testplayer2^.Frame;
      testplayer2^.Draw(0,0,testsurface2);
      testsurface +:= testsurface2;
    END;

//    blur32(testsurface,testsurface,Area(testsurface));

    testconsole^.Copy(0,0,testsurface,Area(testsurface));
    testconsole^.Update;
  UNTIL (keypressed) OR (NOT testplayer^.Active);

  WHILE (keypressed) DO readkey;

  dispose(testplayer,Close);
  dispose(testsurface,Close);

  IF (testplayer2 <> NIL) THEN
  BEGIN
    dispose(testplayer2,Close);
    dispose(testsurface2,Close);
  END;

    // close Console and free that one, too
  testconsole^.Close;
  dispose(testconsole,Destroy);

    // set textmode, and exit
  set_textmode;

  writeln('muu TrueColor library for TMT Pascal - FLIc player example');
  writeln('Copyright (C) 1999 Tor Arne Gustad (anden@online.no)');
end.
