{
   muu TrueColor library for TMT Pascal - Nova 32bpp converter routines
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

PROCEDURE Nx86_S32_D24(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  ax,   [esi]
    mov  bl,   [esi+2]
    mov  [edi],    ax
    mov  [edi+2],  bl

    add  esi,  4
    add  edi,  3
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D16(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // ________ RRRRRRRR GGGGGGGG BBBBBBBB -> RRRRRGGG GGGBBBBB
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    mov  eax,  [esi]

    // ax = _____GGG GGGBBBBB
    shr  ah,   2
    shr  ax,   3

    // ah = RRRRRGGG
    mov  ebx,  eax
    shr  ebx,  8
    and  bh,   $F8
    or   ah,   bh

    mov  [edi],  ax

    add  esi,  4
    add  edi,  2

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D15(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // ________ RRRRRRRR GGGGGGGG BBBBBBBB -> _RRRRRGG GGGBBBBB
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  eax,  [esi]

    // ax = ______GG GGGBBBBB
    shr  ah,   3
    shr  ax,   3
    
    // ah = _RRRRRGG
    mov  ebx,  eax
    shr  ebx,  9
    and  bh,   $7C
    or   ah,   bh

    mov  [edi],  ax

    add  esi,  4
    add  edi,  2

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D8(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  eax,  [esi]
    shr  ah,   6
    shr  ax,   5

    mov  ebx,  eax
    shr  ebx,  16
    and  bl,   $E0
    or   al,   bl
    mov  [edi],  al

    add  esi,  4
    inc  edi

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D32_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    mov  ax,   [esi]
    mov  bx,   [esi+2]
    xchg al,   bl   // swap
    mov  [edi],   ax
    mov  [edi+2], bx
    add  esi,  4
    add  edi,  4
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D24_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  ax,   [esi]
    mov  bx,   [esi+2]
    xchg al,   bl   // swap
    mov  [edi],   ax
    mov  [edi+2], bl
    add  esi,  4
    add  edi,  3
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D16_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  eax,  [esi]

    mov  ebx,  eax
    shr  ebx,  8
    xchg al,   bh   // swap
    shr  ah,   2
    shr  ax,   3
    and  bh,   $F8
    or   ah,   bh

    mov  [edi],  ax

    add  esi,  4
    add  edi,  2

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D15_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  eax,  [esi]

    mov  ebx,  eax
    rol  eax,  16   // swap
    mov  ah,   bh

    shr  ah,   3
    shr  ax,   3
  
    mov  ebx,  eax
    shr  ebx,  9
    and  bh,   $7C
    or   ah,   bh

    mov  [edi],  ax

    add  esi,  4
    add  edi,  2

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D8_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  eax,  [esi]

    mov  ebx,  eax
    rol  eax,  16   // swap
    mov  ah,   bh

    shr  ah,   6
    shr  ax,   5

    mov  ebx,  eax
    shr  ebx,  16
    and  bl,   $E0
    or   al,   bl
    mov  [edi],  al

    add  esi,  4
    inc  edi

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S32_D8_GRAY(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  eax,  [esi]

    shr  al,   2
    shr  ah,   2
    mov  bl,   al
    add  bl,   ah
    shr  eax,  8
    shr  ah,   2
    add  bl,   ah

    mov  [edi],  bl

    add  esi,  4
    inc  edi

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;
