{
   muu TrueColor library for TMT Pascal - Console Wrapper (debugging)
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

UNIT mc_Wrap;

    // Special Console notes
    // ------------------------
    // The wrapper always writes to the standard output (stdout), which means
    // it will default to the screen, but can be easily redirected to all 
    // from a file to a printer. 


INTERFACE
{$R-,I+,Q-}

USES mArea,mClip,mFormat,mSurface,mConsole,Nova,Error;

CONST PixFormatN:   ARRAY[PixelFormat] OF string = ('RGB8888','RGB888',
                                                    'RGB565','RGB555',
                                                    'BGR8888','BGR888',
                                                    'BGR565','BGR555',
                                                    'BGR323','GRAY8',
                                                    'DITHER8','DITHER15',
                                                    'DITHER16','VPalette',
                                                    'Unknown');

TYPE TTYConsole =   ^TTYConsoleT;
     TTYConsoleT =  OBJECT (ConsoleT)
                     
                     public
                        // Console object creation
                      CONSTRUCTOR Create;
                      DESTRUCTOR Destroy; VIRTUAL;

                        // Option interface
                      PROCEDURE Option(optv:pchar); VIRTUAL;

                        // Initialization
                      PROCEDURE Open(title:pchar;lwidth,lheight:word;lformat:Format); VIRTUAL;
                      PROCEDURE Close; VIRTUAL;

                        // Error interface
                      FUNCTION Error:   boolean; VIRTUAL;

                        // Console copy/clear
                      PROCEDURE Copy(tx,ty:longint;m_surface:Surface;m_area:Area); VIRTUAL;
                      PROCEDURE Clear(m_area:Area;m_color:dword); VIRTUAL;
                      PROCEDURE Stretch(m_source:Area;m_surface:Surface;m_dest:Area); VIRTUAL;

                        // Update screen
                      PROCEDURE Update; VIRTUAL;

                        // Console information
                      FUNCTION Info:    ConsoleInfo; VIRTUAL;

                      PROCEDURE Message(m_string:string); VIRTUAL;

                        // Special features
                      PROCEDURE Wrap(wrap_console:Console); VIRTUAL;

                        // internal variables
                     private
                      con_Wrap:     Console;
                      out_File:     text;
                      con_Messages: boolean;
                    END;

IMPLEMENTATION

CONSTRUCTOR TTYConsoleT.Create;
BEGIN
  writeln('- Initializing mTC Console Wrapper...');
  con_Wrap := NIL;

    // assign to standard output
  assign(out_File,'');
  {$I-} rewrite(out_File); {$I+}
  IF (IOResult <> 0) THEN
    fail('Console wrapper:  Failed to open output file');

  writeln(out_File,'info: Wrapper initialized');
END;

DESTRUCTOR TTYConsoleT.Destroy;
BEGIN
  writeln(out_File,'wrap: Console.Destroy;');
  dispose(con_Wrap,Destroy);
  system.close(out_File);
END;

PROCEDURE TTYConsoleT.Option;
BEGIN
  writeln(out_File,'wrap: Console.Option("',optv,'");');
  con_Wrap^.Option(optv);
END;

PROCEDURE TTYConsoleT.Open;
BEGIN
  writeln(out_File,'wrap: Console.Open("',title,'",',lwidth,',',lheight,',',PixFormatN[lformat.pixelformat],');');
  IF (con_Wrap <> NIL) THEN
    con_Wrap^.Open(title,lwidth,lheight,lformat)
  ELSE BEGIN
    writeln(out_File,'info: No Console found');
    system.close(out_File);
    fail('Console Wrapper has no console assigned');
  END;
END;

PROCEDURE TTYConsoleT.Close;
BEGIN
  writeln(out_File,'wrap: Console.Close;');
  con_Wrap^.Close;
  writeln(out_File,'info: Console unwrapped');
END;

FUNCTION TTYConsoleT.Error;
BEGIN
  writeln(out_File,'wrap: Console.Error;');
  result := con_Wrap^.Error;
END;

PROCEDURE TTYConsoleT.Copy;
BEGIN
  writeln(out_File,'wrap: Console.Copy(',tx,',',ty,',Surface[',m_surface^.Width,',',m_surface^.Height,',',PixFormatN[m_surface^.sFormat.pixelformat],'],Area[',m_area.Left,',',m_area.Top,',',m_area.Right,',',m_area.Bottom,']);');
  
  IF (Null_area(m_area)) THEN
    writeln(out_File,'warning: Null area detected');
  
  IF (Unsorted_area(m_area)) THEN
    writeln(out_File,'warning: Unsorted area detected');

  con_Wrap^.Copy(tx,ty,m_surface,m_area);
END;

PROCEDURE TTYConsoleT.Clear;
BEGIN
  writeln(out_File,'wrap: Console.Clear(',m_color,',Area[',m_area.Left,',',m_area.Top,',',m_area.Right,',',m_area.Bottom,']');
  
  IF (Null_area(m_area)) THEN
    writeln(out_File,'warning: Null area detected');
  
  IF (Unsorted_area(m_area)) THEN
    writeln(out_File,'warning: Unsorted area detected');

  con_Wrap^.Clear(m_area,m_color);
END;

PROCEDURE TTYConsoleT.Stretch;
BEGIN
  writeln(out_File,'wrap: Console.Stretch(Area[',m_source.Left,',',m_source.Top,',',m_source.Right,',',m_source.Bottom,'],Surface[',m_surface^.Width,',',m_surface^.Height,',',PixFormatN[m_surface^.sFormat.pixelformat],'],Area[',m_dest.Left,',',m_dest.Top,',',m_dest.Right,',',m_dest.Bottom,']);');
  
  IF (Null_area(m_source)) THEN
    writeln(out_File,'warning: Null area detected (source)');

  IF (Unsorted_area(m_source)) THEN
    writeln(out_File,'warning: Unsorted area detected (source)');

  IF (Null_area(m_dest)) THEN
    writeln(out_File,'warning: Null area detected (destination)');

  IF (Unsorted_area(m_dest)) THEN
    writeln(out_File,'warning: Unsorted area detected (destination)');

  con_Wrap^.Stretch(m_source,m_surface,m_dest);
END;

PROCEDURE TTYConsoleT.Update;
BEGIN
  writeln(out_File,'wrap: Console.Update;');
  con_Wrap^.Update;
END;

FUNCTION TTYConsoleT.Info;
  result := con_Wrap^.Info;

PROCEDURE TTYConsoleT.Message;
BEGIN
  writeln(out_File,'message: ',m_string);
  con_Wrap^.Message(m_string);
END;

PROCEDURE TTYConsoleT.Wrap;
BEGIN
  writeln(out_File,'info: Console assigned to wrapper');
  con_Wrap := wrap_console;
END;

end.
