{
   muu TrueColor library for TMT Pascal - Font writer unit
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

UNIT mFont;

INTERFACE

USES mArea,mClip,mFormat,mSurface,mTarga,mfx32,STRINGS,Error;

TYPE characterlist = ARRAY[byte] OF Surface;

TYPE writemode = (mfNormal,mfStencil,mfAdd,mfSub);

TYPE muuFont = ^muuFontT;
     muuFontT = OBJECT
                  
                 public
                   CONSTRUCTOR Open(script:string);
                   DESTRUCTOR Close;
                   
                   PROCEDURE Assign(chr:byte;fname:string);

                    // write a string
                   PROCEDURE Write(px,py:longint;dest:Surface;txt:string;modep:writemode);

                    // return the pixel length of a string
                   FUNCTION Width(txt:string):  dword;

                 private
                   PROCEDURE get_arg_n(VAR dest,buf:string;arg:dword);
                   PROCEDURE read_dfs(script:string);
                   
                   characters:  characterlist;
                   stencil:     dword;
                END;


IMPLEMENTATION

CONSTRUCTOR muuFontT.Open;
  VAR c:    byte;
BEGIN
  FOR c := 0 TO 255 DO
    characters[c] := NIL;
  
  read_dfs(script);
END;

DESTRUCTOR muuFontT.Close;
  VAR c:    byte;
BEGIN
  FOR c := 0 TO 255 DO
    IF (characters[c] <> NIL) THEN
    BEGIN
      dispose(characters[c],Close);
      characters[c] := NIL;
    END;
END;

PROCEDURE muuFontT.Assign;
BEGIN
  IF (characters[chr] <> NIL) THEN
    dispose(characters[chr],Close);

  characters[chr] := loadTGA(fname,Format(32));
END;

PROCEDURE muuFontT.Write;
VAR pos:    byte := 0;
  CASE (modep) OF
    mfNormal:   FOR pos := 1 TO length(txt) DO
                BEGIN
                  dest^.Copy(px,py,characters[byte(txt[pos])],Area(characters[byte(txt[pos])]));
                  px +:= characters[byte(txt[pos])]^.Width;
                END;

    mfStencil:  FOR pos := 1 TO length(txt) DO
                BEGIN
                  BGCopy(px,py,stencil,characters[byte(txt[pos])],dest);
                  px +:= characters[byte(txt[pos])]^.Width;
                END;
    
    mfAdd:      FOR pos := 1 TO length(txt) DO
                BEGIN
                  AddCopy(px,py,characters[byte(txt[pos])],dest);
                  px +:= characters[byte(txt[pos])]^.Width;
                END;

    mfSub:      FOR pos := 1 TO length(txt) DO
                BEGIN
                  SubCopy(px,py,characters[byte(txt[pos])],dest);
                  px +:= characters[byte(txt[pos])]^.Width;
                END;
  END;

FUNCTION muuFontT.Width;
  VAR pos:  byte := 0;
BEGIN
  result := 0;
  FOR pos := 1 TO length(txt) DO
    result +:= characters[byte(txt[pos])]^.Width;
END;

PROCEDURE muuFontT.get_arg_n;
  VAR bpos,bend,c: byte := 1;
BEGIN
  WHILE (buf[bpos] = ' ') AND (bpos <= length(buf)) DO
    bpos +:= 1;

  WHILE (arg > 0) DO
  BEGIN
    WHILE (buf[bpos] <> ' ') AND (bpos <= length(buf)) DO
      bpos +:= 1;
    WHILE (buf[bpos] = ' ') AND (bpos <= length(buf)) DO
      bpos +:= 1;
    
    arg -:= 1;
  END;

  bend := bpos;
  WHILE (buf[bend] <> ' ') AND (bend <= length(buf)) DO
    bend +:= 1;

  FOR c := 0 TO (bend-bpos) DO
    dest[c+1] := buf[bpos+c];

  dest[0] := char(bend-bpos);
END;

PROCEDURE muuFontT.read_dfs;
  VAR sf:       text;   // scriptfile
      rb,fs:    string; // readbuffer,filestring
      sp,fn:    string; // sprite,filename
      sn:       byte;   // spritenumber
BEGIN
  system.assign(sf,script);
  {$I-}system.reset(sf);{$I+}
  IF (system.IOResult <> 0) THEN
    fail('muuFont:  Unable to open script file');

  WHILE (NOT EOF(sf)) DO
  BEGIN
    readln(sf,rb);

        // skip comments and blank lines
    IF ((rb[1] = '*') AND (rb[2] = '*')) OR
       (length(rb) = 0) THEN
      continue;

        // get character and filename
    get_arg_n(sp,rb,0);
    get_arg_n(fn,rb,1);

    IF (length(sp) = 1) THEN
      sn := byte(sp[1])
    ELSE
      sn := strtoint(sp);

    str(fn,fs);

    Assign(sn,fs);
  END; 

  stencil := 0;

  system.close(sf);
END;


end.
