{
   muu Library for TMT Pascal - Mouse handler object
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

UNIT mouse2K;

INTERFACE
{$R-,I+,Q-}

USES Memory,Error;

TYPE _restrictions = (fixed,relative);
CONST _restrictions_val:array[_restrictions] OF byte = (0,1);

TYPE _buttontypes = (None,Left,Right,Any);
CONST _buttontypes_val:array[_buttontypes] OF byte = (0,1,2,3);

TYPE m2K  = ^m2Kx;
     m2Kx = OBJECT
             private
              _xPos,_yPos:word;
              _xRel,_yRel:integer;
              _xHot,_yHot:word; // cursor hotspot
              _status:byte;
              _buttons:byte;

              FUNCTION Reset:byte;

             public
              CONSTRUCTOR Enable;
              DESTRUCTOR Disable;

              PROCEDURE Hotspot(hotX,hotY:integer);

              PROCEDURE Update;
              PROCEDURE Restrict(xmin,ymin,xmax,ymax:word;rstc:_restrictions);
              PROCEDURE Relative(xcoord,ycoord:integer);
              PROCEDURE Place(xcoord,ycoord:word);
              PROCEDURE Sensitivity(xsense,ysense,dtresh:word);
              PROCEDURE Mickey(xval,yval:word);
              PROCEDURE Graphicsmode(vgamode:word;xcharsize,ycharsize:byte);

              FUNCTION Xpos:integer;
              FUNCTION Ypos:integer;
              FUNCTION Button(btype:_buttontypes):boolean;
            END;

IMPLEMENTATION

FUNCTION m2Kx.Reset; assembler; // ->0 if not installed, else ->n.o.buttons
ASM
  xor ax,  ax
  int 33h
  cmp ax,  00h
  je  @@nomouse
  mov ax,  bx
  jmp @@quit
@@nomouse:
  xor ax,  ax
@@quit:
END;

CONSTRUCTOR m2Kx.Enable;
VAR numB:byte;
    ver:word;
    info:word;
BEGIN
  numB := reset;
  IF (numB=0) THEN
    fail('[m2K]-> No mouse driver found.');

  ASM
    mov  ax,  24h
    int  33h
    mov  [ver],  bx
    mov  [info], cx
  END;

    // set mouse character size to 1x1
  Graphicsmode($13,0,0);

  _buttons := byte(numB);
  _status := 0;
  _xRel := 0;
  _yRel := 0;
  _xHot := 0;
  _yHot := 0;
  update;
END;

DESTRUCTOR m2Kx.Disable; assembler;
  ASM
    mov  ax,  00h
    int  33h
  END;             

PROCEDURE m2Kx.Hotspot;
BEGIN
  _xHot := hotX;
  _yHot := hotY;
END;

PROCEDURE m2Kx.Update;
  VAR mx,my,mb:word;
BEGIN
  ASM
    mov  ax,   03h
    int  33h
    mov  [mx], cx
    mov  [my], dx
    mov  [mb], bx
  END;
  _xPos := mx;
  _yPos := my;
  _status := mb;
END;

PROCEDURE m2Kx.Restrict;
BEGIN
  ASM
    mov  ax,  07h
    mov  cx,  [xmin]
    mov  dx,  [xmax]
    int  33h
    inc  ax
    mov  cx,  [ymin]
    mov  dx,  [ymax]
    int  33h
  END;

  CASE (_restrictions_val[rstc]) OF
    0 : relative(0,0);
    1 : relative(xmin,ymin);
  END;
END;

PROCEDURE m2Kx.Relative;
BEGIN
  _xRel := xcoord;
  _yRel := ycoord;
END;

PROCEDURE m2Kx.Place;
BEGIN
  _xPos := xcoord;
  _yPos := ycoord;
  ASM
    mov  ax,  04h
    mov  cx,  [xcoord]
    mov  dx,  [ycoord]
    int  33h
  END;
END;

PROCEDURE m2Kx.Sensitivity;
BEGIN
  xSense := (xSense MOD 100);
  ySense := (ySense MOD 100);
  ASM
    mov  ax,  1ah
    mov  bx,  [xsense]
    mov  cx,  [ysense]
    mov  dx,  [dtresh]
    and  dx,  32767
    int  33h
  END;
END;

PROCEDURE m2Kx.Mickey; assembler;
ASM
  mov  cx,  xval
  mov  dx,  yval
  mov  ax,  $0f
  int  $33
END;

PROCEDURE m2Kx.Graphicsmode; assembler;
ASM
  mov  cx,  vgamode
  mov  dl,  xcharsize
  mov  dh,  ycharsize
  mov  ax,  $28
  int  $33
END;

FUNCTION m2Kx.Xpos;
  result := _xPos-_xRel;

FUNCTION m2Kx.Ypos;
  result := _yPos-_yRel;

FUNCTION m2Kx.Button;
  result := boolean(_buttontypes_val[btype] AND _status);

end.
